body {
    font-family: 'Verdana', 'Helvetica', 'Arial', sans-serif;
    font-size: 100%;
    background-color: #ffffff;
    color: #000000;
    margin: 0;
    padding: 0;
}

h1 {
    background-color: #3c467e;
    color: #ffffff;
    padding: 5;
}

.ok { background-color: #bfb; } 
.up { background-color: #ffb; } 
.inst  { background-color: #fbb; }
